##############################################################################
#
# Copyright (c) 2005 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Hotfix_20050405 product initialization

Please see README.txt for an explanation of the vulnerability, including
affected Zope versions.

$Id: __init__.py,v 1.2 2005/04/05 17:36:40 tseaver Exp $
"""
# This metaclass supplies the security declarations that allow all
# attributes of a class and its instances to be read and written.
def _patched_metaclass(name, bases, dict):
    for k, v in dict.items():
        if k.endswith('__roles__') and k[:len('__roles__')] not in dict:
            raise Unauthorized, "Can't override security: %s" % k
    ob = type(name, bases, dict)
    ob.__allow_access_to_unprotected_subobjects__ = 1
    ob._guarded_writes = 1
    return ob

def _patched_checkName(self, node, name):
    """Verifies that a name being assigned is safe.

    This is to prevent people from doing things like:

      __metatype__ = mytype (opens up metaclasses, a big unknown
                             in terms of security)
      __path__ = foo        (could this confuse the import machinery?)
      _getattr = somefunc   (not very useful, but could open a hole)

    Note that assigning a variable is not the only way to assign
    a name.  def _badname, class _badname, import foo as _badname,
    and perhaps other statements assign names.  Special case:
    '_' is allowed.
    """
    if name.startswith("_") and name != "_":
        # Note: "_" *is* allowed.
        self.error(node, '"%s" is an invalid variable name because'
                   ' it starts with "_"' % name)
    if name.endswith('__roles__'):
        self.error(node, '"%s" is an invalid variable name because '
                   'it ends with "__roles__".' % name)
    if name == "printed":
        self.error(node, '"printed" is a reserved name.')

def _patched_checkAttrName(self, node):
    """Verifies that an attribute name does not start with _.

    As long as guards (security proxies) have underscored names,
    this underscore protection is important regardless of the
    security policy.  Special case: '_' is allowed.
    """
    name = node.attrname
    if name.startswith("_") and name != "_":
        # Note: "_" *is* allowed.
        self.error(node, '"%s" is an invalid attribute name '
                   'because it starts with "_".' % name)
    if name.endswith('__roles__'):
        self.error(node, '"%s" is an invalid attribute name '
                   'because it ends with "__roles__".' % name)

def initialize(context):

    from AccessControl import ZopeGuards
    from RestrictedPython.RestrictionMutator import RestrictionMutator
    from zLOG import LOG, INFO

    ZopeGuards._metaclass = _patched_metaclass
    ZopeGuards._safe_globals['__metaclass__'] = _patched_metaclass

    RestrictionMutator.checkName = _patched_checkName
    RestrictionMutator.checkAttrName = _patched_checkAttrName

    LOG('Hotfix_20050405', INFO, 'Hotfix applied')
